//+------------------------------------------------------------------+
//|                                               Moving Average.mq4 |
//|                      Copyright  2005, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
#define MAGICMA  20050610

extern bool eSound               = false;
extern double Lots               = 0.1;
extern int eTP                   = 600;
extern int TfMA                  = PERIOD_H1;
extern double MovingPeriod       = 8;
extern double MovingShift        = 0;
extern double MovingPeriodClose  = 24;
extern string es1                = "0 - SMA, 1 - EMA, 2 - SMMA. 3 - LWMA";
extern int ModeMA                = MODE_EMA;
extern double MaximumRisk        = 0.02;
extern double DecreaseFactor     = 3;
//+------------------------------------------------------------------+
//| Calculate open positions                                         |
//+------------------------------------------------------------------+
int CalculateCurrentOrders(string symbol)
  {
   int buys=0,sells=0;
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
      if(OrderSymbol()==Symbol() && OrderMagicNumber()==MAGICMA)
        {
         if(OrderType()==OP_BUY)  buys++;
         if(OrderType()==OP_SELL) sells++;
        }
     }
//---- return orders volume
   if(buys>0) return(buys);
   else       return(-sells);
  }
//+------------------------------------------------------------------+
//| Calculate optimal lot size                                       |
//+------------------------------------------------------------------+
double LotsOptimized()
  {
   double lot=Lots;
   int    orders=HistoryTotal();     // history orders total
   int    losses=0;                  // number of losses orders without a break
//---- select lot size
   lot=NormalizeDouble(AccountFreeMargin()*MaximumRisk/1000.0,1);
//---- calcuulate number of losses orders without a break
   if(DecreaseFactor>0)
     {
      for(int i=orders-1;i>=0;i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false) { Print("Error in history!"); break; }
         if(OrderSymbol()!=Symbol() || OrderType()>OP_SELL) continue;
         //----
         if(OrderProfit()>0) break;
         if(OrderProfit()<0) losses++;
        }
      if(losses>1) lot=NormalizeDouble(lot-lot*losses/DecreaseFactor,1);
     }
//---- return lot size
   if(lot<0.1) lot=0.1;
   return(lot);
  }
//+------------------------------------------------------------------+
//| Check for open order conditions                                  |
//+------------------------------------------------------------------+
void CheckForOpen()
  {
   double ma,maprev,tp=0;
   int    res;
//---- go trading only for first tiks of new bar
   if(Volume[0]>1) return;
//---- get Moving Average 
   ma=iMA(NULL,TfMA,MovingPeriod,MovingShift,ModeMA,PRICE_CLOSE,0);
   maprev=iMA(NULL,TfMA,MovingPeriod,MovingShift,ModeMA,PRICE_CLOSE,1);
//---- sell conditions
   if(Open[1]>Close[1] && ma<maprev)  
     {
       if (eTP>0) tp=Bid-eTP*Point;
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,0,tp,"",MAGICMA,0,Red);
      if (res>0) {
        if (eSound) PlaySound("ok.wav");
      }
      return;
     }
//---- buy conditions
   if(Open[1]<Close[1] && ma>maprev)  
     {
      if (eTP>0) tp=Ask+eTP*Point;
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,0,tp,"",MAGICMA,0,Blue);
      if (res>0) {
        if (eSound) PlaySound("ok.wav");
      }
      return;
     }
//----
  }
//+------------------------------------------------------------------+
//| Check for close order conditions                                 |
//+------------------------------------------------------------------+
void CheckForClose()
  {
   double ma;
//---- go trading only for first tiks of new bar
   if(Volume[0]>1) return;
//---- get Moving Average 
   ma=iMA(NULL,TfMA,MovingPeriodClose,MovingShift,ModeMA,PRICE_CLOSE,0);
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false)        break;
      if(OrderMagicNumber()!=MAGICMA || OrderSymbol()!=Symbol()) continue;
      //---- check order type 
      if(OrderType()==OP_BUY)
        {
         if(Open[1]>ma && Close[1]<ma) {
           OrderClose(OrderTicket(),OrderLots(),Bid,3,White);
           if (eSound) PlaySound("alert2.wav");
         }
         break;
        }
      if(OrderType()==OP_SELL)
        {
         if(Open[1]<ma && Close[1]>ma) {
           OrderClose(OrderTicket(),OrderLots(),Ask,3,White);
           if (eSound) PlaySound("alert2.wav");
         }
         break;
        }
     }
//----
  }
//+------------------------------------------------------------------+
//| Start function                                                   |
//+------------------------------------------------------------------+
void start()
  {
//---- check for history and trading
   if(Bars<100 || IsTradeAllowed()==false) return;
//---- calculate open orders by current symbol
   if(CalculateCurrentOrders(Symbol())==0) CheckForOpen();
   else                                    CheckForClose();
//----
  }
//+------------------------------------------------------------------+